// ASSOCIATIVE ARRAY HEADER FILE for GNU g++
// AssocArray<value> key = allways char*, value is any data type 

#ifndef _gassoc_h_
#define _gassoc_h_ 1

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>
#include "ulist.h"

#define ASOC_HASH 1021
#define MAT_DEL_CHR ":"
#define MAT_CHR_LEN 1024

typedef ulist<char*>	AssocKeyList ;

template<class T> class AssocArray ;
template<class T> class StringDataBase ;

template<class T>
class AssocList {
	friend class AssocArray<T> ;
	friend class StringDataBase<T> ;
	protected :
		char      *key ;
		T          val ;
		AssocList *left ;
		AssocList *right ;

	public :
		AssocList() { key = NULL; left = NULL; right = NULL; }
		~AssocList(){}
} ;

template<class T>
class AssocArray {
	protected :
		AssocList<T>* hashtable[ASOC_HASH] ;
		AssocList<T>  aa_nil ;
		int hashf(char *str)
		{
			unsigned val ;

			for (val=0; *str != '\0'; str++)
				val = ((val << CHAR_BIT) + *str) % ASOC_HASH ;
	
			return (int) val ;
		}

	public :
		AssocArray() {initial(); }
		~AssocArray(){}

		AssocKeyList  keylist ;

		void initial()
		{
			for (int i=0; i<ASOC_HASH; i++)
				hashtable[i] = &aa_nil ;
			keylist.clear() ;
		}
		int iskey(char *key)
		{
			int cmp ;
			AssocList<T> **p ;

			p = &hashtable[hashf(key)] ;

			while ((*p) != &aa_nil) {
				if ((cmp = strcmp((*p)->key, key)) == 0) {
					return 1 ;
				}
				else {
					if (cmp < 0) p = &((*p)->left) ;
					else         p = &((*p)->right) ;
				}
			}

			return 0 ;
		}
		T&  operator [] (char *key)
		{
			int cmp ;
			AssocList<T> **p, *n ;

			p = &hashtable[hashf(key)] ;

			while ((*p) != &aa_nil) {
				if ((cmp = strcmp((*p)->key, key)) == 0) {
					return (*p)->val ;
				}
				else {
					if (cmp < 0) p = &((*p)->left) ;
					else         p = &((*p)->right) ;
				}
			}

			n = new AssocList<T>[1] ;
			n->key = new char[strlen(key)+1] ;
			strcpy(n->key, key) ;
			n->left  = &aa_nil ;
			n->right = &aa_nil ;
			(*p) = n ;

			keylist.append(n->key) ;

			return n->val ;
		}
		char* key_ptr (char *key)
		{
			int cmp ;
			AssocList<T> **p ;

			p = &hashtable[hashf(key)] ;

			while ((*p) != &aa_nil) {
				if ((cmp = strcmp((*p)->key, key)) == 0) {
					return key ;
				}
				else {
					if (cmp < 0) p = &((*p)->left) ;
					else         p = &((*p)->right) ;
				}
			}

			return 0 ;
		}
} ;
template<class T>
class AssocMatrix {
        protected :
                char          testbuf[MAT_CHR_LEN] ;

        public :
                AssocArray<T> matrix ;
                AssocMatrix() {}
                ~AssocMatrix(){}

                int iskeys(char *ka, char *kb) ;
                AssocKeyList* get_keylist() ;
                T& operator () (char *ka, char *kb) ;
} ;

template<class T>
int AssocMatrix<T>::iskeys(char *ka, char *kb)
{
        strcpy(testbuf, ka) ;
        strcat(testbuf, MAT_DEL_CHR) ;
        strcat(testbuf, kb) ;

        return matrix.iskey(testbuf) ;
}

template<class T>
T& AssocMatrix<T>::operator () (char *ka, char *kb)
{
        strcpy(testbuf, ka) ;
        strcat(testbuf, MAT_DEL_CHR) ;
        strcat(testbuf, kb) ;

        return matrix[testbuf] ;
}

template<class T>
AssocKeyList* AssocMatrix<T>::get_keylist()
{
        return &matrix.keylist ;
}

typedef	ulist<char*>	StringList ;

template<class T>
class StringDataBase : public AssocArray<T>
{
	public:
	StringDataBase() {}
	~StringDataBase(){}

	// adaptive functions
	inline int find(char *str)	{ return AssocArray<T>::iskey(str) ; }
	char*	entry(char* str, T cnt)
	{
		int cmp ;
		AssocList<T> **p, *n ;

		p = &hashtable[hashf(str)] ;

		while ((*p) != &aa_nil) {
			if ((cmp = strcmp((*p)->key, str)) == 0) {
				(*p)->val = cnt ;
				return (*p)->key ;
			}
			else {
				if (cmp < 0) p = &((*p)->left) ;
				else         p = &((*p)->right) ;
			}
		}

		n = new AssocList<T> ;
		n->key = str ;
		n->left = &aa_nil ;
		n->right = &aa_nil ;
		n->val = cnt ;

		(*p) = n ;

		keylist.append(str) ;

		return str ;
	}
	T		pickup(char* str)
	{
		int cmp ;
		AssocList<T> **p, *n ;

		p = &hashtable[hashf(str)] ;

		while ((*p) != &aa_nil) {
			if ((cmp = strcmp((*p)->key, str)) == 0) {
				return (*p)->val ;
			}
			else {
				if (cmp < 0) p = &((*p)->left) ;
				else         p = &((*p)->right) ;
			}
		}

		return (T)0 ;
	}
} ;

#endif
