//
// NLD-NetList -> XNF-NetList File Converter for XC3000/XC3100/XC4000
//  first products is perl script written by Ta.Fujii (nld_xnf.pl Ver.1.8)
//  this program is rewritenn with C++ code by A.Tsutsui
//
//  Written and Revised by Akihiro Tsutsui
//  Copyright 1993 NTT All rights reserved
//

/* History ..
 VERSION 1.0  93/01/27 Compatible tatsuya's nld_xnf.pl
 VERSION 1.1  93/02/01 expart mode support & customize
 VERSION 1.2  93/02/03 expart mode support & customize
 VERSION 1.3  93/02/19 OBUF FAST flag support for xc4000
 VERSION 2.0a 93/07/27 code rewrite and major change
 VERSION 2.0b 93/08/13 INFF OUTFF auto add mode
 VERSION 2.1a 93/11/08 command file parser included, -e option abolished
					   after this version, libg++ is indispensable
 VERSION 2.1b 93/11/18 -e option revive, GSR reset method supported..
 VERSION 2.1c 94/03/24 bug fix for iob latch activation
 VERSION 2.5  94/04/22 Major Change of Spec
                       - cmp_xrf.tbl conversion table is not required
                       - option -m does not effect
                       + def-special name xrf description support
                       + def-special HARD_MACRO / SOFT_MACRO
                       + def-special Memory Information support
                       + option -F flush registor to IOFF (direct connect only)
						 (This function killed currently.)
 VERSION 2.51 95/06/01 Support both UNIX and MS-DOS environment
 VERSION 2.52 95/07/01 Using new version of parsers
*/
 
#include <iostream.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>
#include <time.h>

#include "nlddefs.h"
#include "gassoc.h"
#include "my_func.h"
#include "nld_xnf.h"

// Credit Definitions

char* const program_name = "nld_xnf" ;
char* const program_version = "2.52 22 Apr '94" ;
char* const program_author = "written and revised by A.Tsutsui\n(C)Copyright by NTT 1993 All rights reserved" ;

// Constants Definitions

#define RET_ERR -1
#define MAX_STR_LEN 256

main(int ac, char** av)
{

	// FILE Defs.

	FILE *fpn ;
	FILE *fpp ;
	FILE *fpo ;
	FILE *fpx ;

	// Iterator & temporal Vars.

	int i ;
	int j ;
	char* tmp_str ;
	char* tmp_buf ;
	char* top_mod_file ;
	char str_buf[MAX_STR_LEN] ;
	char name_buf[MAX_STR_LEN] ;
	char nld_mod_file[MAX_STR_LEN] ;
	char pcd_pth_name[MAX_STR_LEN] ;

	// List Data Structures

	ulist<char*> pcd_path_list ;
	ulist<char*> inp_pin_stack ;

	// String Database Data Structures
  
	StringDataBase<NLD_PCD_MODULE*> mod_clas_tbl ;
	StringDataBase<int>				cmp_clas_lst ;
	StringDataBase<int>				cmp_clas_atr ;
	StringDataBase<NLDPinType>		top_mod_pins ;
	StringDataBase<int>				pin_net_num ;
	StringDataBase<int>				cmp_net_num ;

	// Associative Arrays

	AssocMatrix<int>		net_num_xrf ;
	AssocMatrix<PCDPinType>	pcd_pin_ref ;

	// NLD PCD Data Structures

	NLD_MODULE_LIST NML_TOP ;
	NLD_MODULE_LIST NML ;
	PCD_MODULE_LIST PML ;
	NLD_MODULE* top_module ;
	NLD_MODULE* nmodule ;
	PCD_MODULE* pmodule ;

	NLDPinInfo* nld_pinfo ;
	NLDCmpInfo* nld_cinfo ;
	NLDNetInfo* nld_ninfo ;
	NLDStrList* nld_slist ;

	NLDNetItem*	nld_nitem ;

	PCDPinInfo* pcd_pinfo ;
	PCDStrList* pcd_slist ;

	NLD_PCD_MODULE np_nil ; // MODULE UNION NIL PTR

	// Functions ..

	int parse_nldx(FILE*, NLD_MODULE_LIST&) ;
	int parse_pcdx(FILE*, PCD_MODULE_LIST&) ;
	void print_usage() ;
	void write_jt(FILE*, int) ;
	void write_inclock(FILE*, int) ;

	// Command Line Options

	char optchar ;
	char part_type[20] ;
	char io_prefix[MAX_STR_LEN] ;
	char out_fname[MAX_STR_LEN] ;
	char xrf_fname[MAX_STR_LEN] ;
	char cmp_xfile[MAX_STR_LEN] ;

	int speed_rate ;
	int clock_rate = 0 ;
	int pullup_set = 0 ;
	int ioset_fast = 0 ;
	int p_reset_on = 0 ;
	int use_gsr_on = 0 ;
	int use_gsr_ng = 1 ;
	int use_gsr_sc = 0 ;
	int sbclock_on = 0 ;
	int rb_jtag_on = 0 ;
	int macro_used = 0 ;
	int flush_regs = 0 ; 
	int presv_name = 0 ;
	int optim_circ = 0 ;
/*	int	ioff_enabl = 0 ; */
	int iob_hi_pin = 0 ;
	int	inpff_on   = 0 ;
	int	outff_on   = 0 ;

	int device_is  = 3 ; // 3=3000 4=4000 5=3100

	cerr << program_name \
		<< " version " << program_version \
		<< " " << program_author << endl ;

	strcpy(part_type, "3042PG132") ;

	io_prefix[0] = '\0' ;
	out_fname[0] = '\0' ;
	xrf_fname[0] = '\0' ;

	int	arg_flg = 0 ;

	while (--ac > 0 && (*++av)[0] == '-') {
		arg_flg = 0 ;
		while (optchar = *++av[0]) {
			switch (optchar) {
			case 'd' :
				if (ac == 1) {
					cerr <<  "Error : Option argument is illegal" << endl ;
					print_usage() ;
					exit(1) ;
				}
				strcpy(part_type, *(++av)) ;
				if (*part_type == '-') {
					cerr <<  "Error : Option argument is illegal" << endl ;
					print_usage() ;
					exit(1) ;
				}
				str_to_upper(part_type) ;
				ac-- ;
				arg_flg = 1 ;
				break ;
			case 's' :
				if (ac == 1) {
					cerr <<  "Error : Option argument is illegal" << endl ;
					print_usage() ;
					exit(1) ;
				}
				speed_rate = atoi(*(++av)) ;
				ac-- ;
				arg_flg = 1 ;
				break ;

			case 'e' :
				if (ac == 1) {
					cerr <<  "Error : Option argument is illegal" << endl ;
					print_usage() ;
					exit(1) ;
				}
				strcpy(io_prefix, *(++av)) ;
				if (*io_prefix == '-') {
					cerr <<  "Error : Option argument is illegal" << endl ;
					print_usage() ;
					exit(1) ;
				}
				ac-- ;
				arg_flg = 1 ;
				break ;
/*
			case 'c' :
				if (ac == 1) {
					cerr << "Error : Optional argument is illegal" << endl ;
					print_usage() ;
					exit(1) ;
				}
				strcpy(cmd_fname, *(++av)) ;
				if (*cmd_fname == '-') {
					cerr << "Error : Optional argument is illegal" << endl ;
					print_usage() ;
					exit(1) ;
				}
				ac-- ;
				arg_flg = 1 ;
				break ;
*/
			case 'i' :
				if (ac == 1) {
					cerr <<  "Error : Option argument is illegal" << endl ;
					print_usage() ;
					exit(1) ;
				}
				str_to_upper(*(av+1)) ;
				if (strcmp(*(av+1), "15") == 0) {
				clock_rate = 15 ;
				} else if (strcmp(*(av+1), "15K") == 0) {
				clock_rate = 150 ;
				} else if (strcmp(*(av+1), "500K") == 0) {
				clock_rate = 500 ;
				} else if (strcmp(*(av+1), "8M") == 0) {
				clock_rate = 8 ;
				} else {
				err_handl(1, "illegal internal osc clock") ;
				}
				av++ ;
				ac-- ;
				arg_flg = 1 ;
				break ;
			case 'o' :
				if (ac == 1) {
					cerr <<  "Error : Option argument is illegal" << endl ;
					print_usage() ;
					exit(1) ;
				}
				strcpy(out_fname, *(++av)) ;
				if (*out_fname == '-') {
					cerr <<  "Error : Option argument is illegal" << endl ;
					print_usage() ;
					exit(1) ;
				}
				ac-- ;
				arg_flg = 1 ;
				break ;
			case 'x' :
				if (ac == 1) {
					cerr <<  "Error : Option argument is illegal" << endl ;
					print_usage() ;
					exit(1) ;
				}
				strcpy(xrf_fname, *(++av)) ;
				if (*xrf_fname == '-') {
					cerr <<  "Error : Option argument is illegal" << endl ;
					print_usage() ;
					exit(1) ;
				}
				ac-- ;
				arg_flg = 1 ;
				break ;
			case 'p' :
				pullup_set = 1 ;
				break ;
			case 'f' :
				ioset_fast = 1 ;
				break ;
			case 'r' :
				p_reset_on = 1 ;
				break ;
			case 'R' :
				use_gsr_on = 1 ;
				switch (*++av[0]) {
					case '+' :
						use_gsr_ng = 0 ;
						break ;
					case 'S' :
						use_gsr_sc = 1 ;
						switch (*++av[0]) {
						case '+' :
							use_gsr_ng = 0 ;
							break ;
						default :
							err_handl(0, "p_reset active logic forced to negative") ;
							use_gsr_ng = 1 ;
							break ;
						}
						break ;
					default :
						err_handl(0, "p_reset active logic forced to negative") ;
						use_gsr_ng = 1 ;
						break ;
				}
				break ;
			case 'b' :
				sbclock_on = 1 ;
				break ;
			case 't' :
				rb_jtag_on = 1 ;
				break ;
			case 'm' :
				macro_used = 1 ;
				break ;
			case 'l' :
				/*
				flush_regs = 1 ;
				ioff_enabl = 1 ;
				*/
				if (ac == 1) {
					cerr <<  "Error : Option argument is illegal" << endl ;
					print_usage() ;
					exit(1) ;
				}
				strcpy(str_buf, *(++av)) ;
				if (*str_buf == '-') {
					cerr <<  "Error : Option argument is illegal" << endl ;
					print_usage() ;
					exit(1) ;
				}
				switch (*str_buf) {
				case 'I' :
				case 'i' :
					inpff_on = 1 ;
					break ;
				case 'O' :
				case 'o' :
					outff_on = 1 ;
					break ;
				case 'B' :
				case 'b' :
					inpff_on = 1 ;
					outff_on = 1 ;
					break ;
				default :
					cerr <<  "Error : Option argument is illegal" << endl ; 
					print_usage() ;
					exit(1) ;
				}
				ac-- ;
				arg_flg = 1 ;
				break ;
			case 'n' :
				presv_name = 1 ;
				break ;
			case 'Z' :
				iob_hi_pin = 1 ;
				break ;
			case 'F' :
				flush_regs = 1 ;
				break ;
			/*
			case '?' :
				print_usage() ;
				exit(0) ;
				break ;
			*/
			default :
				err_handl(1, "illegal command line option") ;
			}
			if (arg_flg) break ;
		}
	}

	if (ac < 1) {
		print_usage() ;
		exit(1) ;
	}
	else if (ac == 1 && !getenv("LCA_PCD_PATH")) {
		print_usage() ;
		exit(1) ;
	}

	// PART TYPE DECODE

	switch (*part_type) {
	case '3' :
		if (*(part_type+1) == '0') { // xc3000
			switch (speed_rate) {
			case 70 :
				break ;
			case 100 :
				break ;
			case 125 :
				break ;
			case 150 :
				break ;
			default :
				err_handl(0, "device speed forced to 125MHz") ;
				speed_rate = 125 ;
				device_is = 3 ;
				break ;
			}
		}
		else if (*(part_type+1) == '1') { // xc3100
			switch (speed_rate) {
			case 3 :
				break ;
			case 4 :
				break ;
			case 5 :
				break ;
			default :
				err_handl(0, "device speed forced to 5ns version") ;
				speed_rate = 5 ;
				device_is = 5 ;
				break ;
			}
		}
		else {
			err_handl(1, "unknown device type") ;
		}
		break ;
	case '4' :
		switch (speed_rate) {
		case 2 :
			break ;
		case 5 :
			break ;
		case 6 :
			break ;
		default :
			err_handl(0, "device speed forced to 5ns") ;
			speed_rate = 5 ;
			break ;
		}
		device_is = 4 ;
		break ;
	default :
		err_handl(1, "unknown device type") ;
	}

	if (device_is != 4) {
		if (rb_jtag_on) {
			err_handl(0, "read back & jtag net is only xc4000's feature.. ignored.") ;
			rb_jtag_on = 0 ;
		}
		if (clock_rate) {
			err_handl(0, "internal clock is only xc4000's feature.. ignored.") ;
			clock_rate = 0 ;
		}
		if (ioset_fast) {
			err_handl(0, "output through rate control is only xc4000's feature.. ignored.") ;
			ioset_fast = 0 ;
		}
		if (macro_used) {
			err_handl(0, "hard macro definition is only xc4000's feature.. ignored.") ;
			macro_used = 0 ;
		}
		if (use_gsr_on) {
			err_handl(0, "GSR module is only xc4000's feature.. ignored.") ;
			use_gsr_on = 0 ;
		}
		if (iob_hi_pin) {
			err_handl(0, "GTS module is only xc4000's feature.. ignored.") ;
			iob_hi_pin = 0 ;
		}
	}

	if (p_reset_on && use_gsr_on) {
		err_handl(0, "p_reset pin conflict.. force to GSR pin.") ;
		p_reset_on = 0 ;
	}

	// Component Cross Ref File (<- X)
	// Component Cross Ref Table

	Xrf_Tbl cmp_name_tbl ;

	// cross-ref file

	int xrf_file_on = 0 ;

	if (*xrf_fname) {
		if ((fpx = fopen(xrf_fname, "wt")) == NULL) {
			sprintf(str_buf, "cannot open xref file %s", xrf_fname) ;
			err_handl(1, str_buf) ;
		}
		xrf_file_on = 1 ;
	}

	top_mod_file = dup_str(*av) ;

	while (*(++av)) pcd_path_list.append(*av) ;

	if (getenv("LCA_PCD_PATH")) {
		tmp_str = dup_str(getenv("LCA_PCD_PATH")) ;
		pcd_path_list.append(tmp_str) ;
		for (i=0; tmp_str[i] != '\0'; i++)
			if (tmp_str[i] == ':') {
				pcd_path_list.append(tmp_str+i+1) ;
				tmp_str[i] = '\0' ;
			}
	}

	if ((fpn = fopen(top_mod_file, "rt")) == NULL) {
		sprintf(str_buf, "cannot open %s", top_mod_file) ;
		err_handl(1, str_buf) ;
	}

	if (parse_nldx(fpn, NML_TOP) == RET_ERR) {
		sprintf(str_buf, "nld file %s parse faild", top_mod_file) ;
		err_handl(1, str_buf) ;
	}

	top_module = NML_TOP.get_first() ; // for first nld module only
  
	forall_plist_items(nld_cinfo, top_module->cmplist) {
		if (!cmp_clas_lst.iskey(nld_cinfo->clas))
			cmp_clas_lst.entry(nld_cinfo->clas, 1) ;
	}
	forall_plist_items(nld_pinfo, top_module->pinlist) {
		sw_bracket(nld_pinfo->name) ;
	}
  
	NLD_PCD_MODULE* tmp_npm ;
	int nld_mod_num = 0 ;
	int pcd_mod_num = 0 ;
	int flag = 0 ;
	int sflag ;

	forall_list_items(tmp_str, cmp_clas_lst.keylist) {
		flag = 0 ;
		forall_list_items(tmp_buf, pcd_path_list) {
			sprintf (name_buf, "%s/%s.pcd", tmp_buf, tmp_str) ;
			if ((fpp = fopen(name_buf, "rt")) != NULL) {
				if (parse_pcdx(fpp, PML) == RET_ERR) {
					sprintf(str_buf, "pcd/nld file %s parse faild",
						name_buf) ;
					err_handl(1, str_buf) ;
				}
				fclose(fpp) ;
				pmodule = PML.get_last() ;
				forall_plist_items(pcd_pinfo, pmodule->pinlist) {
					sw_bracket(pcd_pinfo->name) ;
				}
//				tmp_npm = new NLD_PCD_MODULE(IS_PCD_MOD, pmodule) ;
				tmp_npm = new NLD_PCD_MODULE(pmodule) ;
				mod_clas_tbl.entry(tmp_str, tmp_npm) ;

				// This feature added at 1994/04/22

				if ((sflag = decode_spf(pmodule->defspec, tmp_str, cmp_name_tbl)) == -1) {
					exit(1) ;
				}
				cmp_clas_atr.entry(tmp_str, sflag) ;

				flag = 1 ;
				pcd_mod_num++ ;
			}
			else {
				sprintf (name_buf, "%s/%s.nld", tmp_buf, tmp_str) ;
				if ((fpp = fopen(name_buf, "rt")) != NULL) {
					if (parse_nldx(fpp, NML) == RET_ERR) {
						sprintf(str_buf, "pcd/nld file %s parse faild",
							name_buf) ;
						err_handl(1, str_buf) ;
					}
					fclose (fpp) ;
					nmodule = NML.get_last() ;
					forall_plist_items(nld_pinfo, nmodule->pinlist) {
						sw_bracket(nld_pinfo->name) ;
					}
//					tmp_npm = new NLD_PCD_MODULE(IS_NLD_MOD, nmodule) ;
					tmp_npm = new NLD_PCD_MODULE(nmodule) ;
					mod_clas_tbl.entry(tmp_str, tmp_npm) ;
					flag = 1 ;
					nld_mod_num++ ;
				}
			}
		}
		if (!flag) {
			sprintf(str_buf, "nld/pcd module %s is not found in serch path", tmp_str) ;
			err_handl(1, str_buf) ;
		}
	}

	forall_plist_items(nld_ninfo, top_module->netlist) {
		forall_list_items(nld_nitem, nld_ninfo->nlink) {
			if (nld_nitem->etyp == IS_PIN) {
				sw_bracket(nld_nitem->signal) ; // Replace [] -> <>
				pin_net_num.entry(nld_nitem->signal, nld_ninfo->id+1) ;
			}
			else if (nld_nitem->etyp == IS_CMP) {
				sw_bracket(nld_nitem->sigpin) ; // Replace [] -> <>
				net_num_xrf(nld_nitem->signal, nld_nitem->sigpin)
					=  nld_ninfo->id + 1;
			}
		}
	}

	// I/O Flip-Flop Operation
/*
	if (ioff_enabl) {
		forall_plist_items(nld_cinfo, top_module->cmplist) {
			if (!strcmp(nld_cinfo->clas, "inff") {
				ioff_comp.entry(nld_cinfo->name, 1) ;
			}
			else if (!strcmp(nld_cinfo->clas, "outff") {
				ioff_comp.entry(nld_cinfo->name, 1) ;
			}
		}
		forall_plist_items(nld_ninfo, top_module->netlist) {
			forall_list_items(nld_nitem, nld_ninfo->nlink) {
				if (ioff_comp.find(nld_nitem->signal)) {
				}
			}
		}
	}
*/
	// Write Header

	if ((fpo = fopen(out_fname, "wt")) == NULL) {
		fpo = stdout ;
	}

	char DayTime[50] ;
	time_t tp ;
	time(&tp) ;
	strftime(DayTime, 50, "%a %b %d %H:%M:%S %Z %Y", localtime(&tp)) ;
	fprintf(fpo, "LCANET, 4\n") ;
	fprintf(fpo, "PROG, %s, %s, Created on %s, Created from %s\n",
		program_name, program_version, DayTime, top_mod_file) ;
	fprintf(fpo, "PART, %s-%d\n", part_type, speed_rate) ;

	// Write XC4000 Optional Network

	if (clock_rate)
		write_inclock(fpo, clock_rate) ;
	if (rb_jtag_on)
		write_jt(fpo, p_reset_on) ;

	// Write I/O Buffers

	int mc_net = 0 ;
	int mc_det = 0 ;
	int mc_unu = 0 ;
	int pr_net ;
	int gnd_net ;
	int vcc_net ;

	NLDPinInfo* mc_pinfo ;

	forall_plist_items(nld_pinfo, top_module->pinlist) {
		if (!strcmp(nld_pinfo->name, "m_clock")) {
			mc_net = pin_net_num.pickup(nld_pinfo->name) ;
			mc_det = 1 ;
			mc_pinfo = nld_pinfo ;
			if (nld_pinfo->type == P_UNUSED) {
				mc_unu = 1 ;
			}
		}
	}

	if ((inpff_on || outff_on) && !mc_net) {
		cerr << "Warning : You are going to use IOB-FF in the circuit without m_clock." << endl ;
		if (mc_det) {
			if (mc_unu) {
				mc_pinfo->type = P_INPUT ;
			}
			else {
				cerr << "Error : isolated m_clock pin" << endl ;
				exit(1) ;
			}
		}
		else {
			cerr << "Warning : m_clock pin is not defined, check your system" ;
			cerr << endl ;
			mc_pinfo = new NLDPinInfo(-1, "m_clock", P_INPUT) ;
			top_module->pinlist->append(mc_pinfo) ;
		}
	}

	forall_plist_items(nld_pinfo, top_module->pinlist) {
		if (nld_pinfo->type == P_UNUSED 
			|| nld_pinfo->type == P_UNUSED_CONNECTED)
			continue ;
		if (nld_pinfo->type == P_UNDEFINED) {
			sprintf(str_buf, "undefined top module pin %s not generated",
				nld_pinfo->name) ;
			err_handl(0, str_buf) ;
			continue ;
		}
		if (nld_pinfo->type == P_INPUT_CONNECTED
			|| nld_pinfo->type == P_OUTPUT_CONNECTED
			|| nld_pinfo->type == P_BIDIRECT_CONNECTED
			|| nld_pinfo->type == P_TRISTATE_CONNECTED
			) {
			sprintf(str_buf, "%s is i/o connected pin. please use io_buf",
				nld_pinfo->name) ;
			err_handl(1, str_buf) ;
		}
		if (!strcmp(nld_pinfo->name, "m_clock")) {
			fprintf(fpo, "SYM, ") ;
			if (device_is == 4) {
				fprintf(fpo, "BUFGP__%d, BUFGP\n", nld_pinfo->id+1) ;
			}
			else {
				fprintf(fpo, "GCLK__%d, GCLK\n", nld_pinfo->id+1) ;
			}
//			mc_net = pin_net_num.pickup(nld_pinfo->name) ;
			fprintf(fpo, "PIN, I, I, %s\n", nld_pinfo->name) ;
			fprintf(fpo, "PIN, O, O, NET__%d\n", mc_net) ;
			fprintf(fpo, "END\n") ;
		} else if (!strcmp(nld_pinfo->name, "s_clock")) {
			if (sbclock_on) {
				fprintf(fpo, "SYM, ") ;
				if (device_is == 4) {
					fprintf(fpo, "BUFGP__%d, BUFGP\n", nld_pinfo->id+1) ;
				}
				else {
					fprintf(fpo, "ACLK__%d, ACLK\n", nld_pinfo->id+1) ;
				}
				fprintf(fpo, "PIN, I, I, %s\n", nld_pinfo->name) ;
				fprintf(fpo, "PIN, O, O, NET__%d\n",
					pin_net_num.pickup(nld_pinfo->name)) ;
				fprintf(fpo, "END\n") ;
			}
		} else if (!strcmp(nld_pinfo->name, "b_clock")) {
			if (sbclock_on) {
				fprintf(fpo, "SYM, ") ;
				if (device_is == 4) {
					fprintf(fpo, "BUFGP__%d, BUFGP\n", nld_pinfo->id+1) ;
				}
				else {
					fprintf(fpo, "IBUF__%d, IBUF\n", nld_pinfo->id+1) ;
				}
				fprintf(fpo, "PIN, I, I, %s\n", nld_pinfo->name) ;
				fprintf(fpo, "PIN, O, O, NET__%d\n",
					pin_net_num.pickup(nld_pinfo->name)) ;
				fprintf(fpo, "END\n") ;
			}
		} else if (!strcmp(nld_pinfo->name, "p_reset")) {
			if (p_reset_on) {
				fprintf(fpo, "SYM, ") ;
				if (device_is == 4) {
					fprintf(fpo, "BUFGP__%d, BUFGP\n", nld_pinfo->id+1) ;
				}
				else {
					fprintf(fpo, "IBUF__%d, IBUF\n", nld_pinfo->id+1) ;
				}
				pr_net = pin_net_num.pickup(nld_pinfo->name) ;
				fprintf(fpo, "PIN, I, I, %s\n", nld_pinfo->name) ;
				fprintf(fpo, "PIN, O, O, NET__%d\n", pr_net) ;
				fprintf(fpo, "END\n") ;
			}
		} else if (!strcmp(nld_pinfo->name, "VSS")) {
			nld_pinfo->type = P_UNUSED ;
			gnd_net = pin_net_num.pickup(nld_pinfo->name) ;
		} else if (!strcmp(nld_pinfo->name, "VDD")) {
			nld_pinfo->type = P_UNUSED ;
			vcc_net = pin_net_num.pickup(nld_pinfo->name) ;
		} else {
			if (nld_pinfo->type == P_OUTPUT) {
				if (outff_on) {
				fprintf(fpo, "SYM, OUTFF__%d, OUTFF\n", nld_pinfo->id+1) ;
				fprintf(fpo, "PIN, D, I, NET__%d\n",
					pin_net_num.pickup(nld_pinfo->name)) ;
				fprintf(fpo, "PIN, Q, O, %s%s\n", io_prefix, nld_pinfo->name) ;
				fprintf(fpo, "PIN, C, I, NET__%d\n", mc_net) ;
				fprintf(fpo, "END\n") ;
				}
				else {
				if (ioset_fast)
					fprintf(fpo, "SYM, OBUF__%d, OBUF, FAST\n", nld_pinfo->id+1) ;
				else
					fprintf(fpo, "SYM, OBUF__%d, OBUF\n", nld_pinfo->id+1) ;
				fprintf(fpo, "PIN, I, I, NET__%d\n",
					pin_net_num.pickup(nld_pinfo->name)) ;
				fprintf(fpo, "PIN, O, O, %s%s\n", io_prefix, nld_pinfo->name) ;
				fprintf(fpo, "END\n") ;
				}
			}
			else if (nld_pinfo->type == P_INPUT || nld_pinfo->type == P_GLOBAL) {
			if (inpff_on) {
				fprintf(fpo, "SYM, INFF__%d, INFF\n", nld_pinfo->id+1) ;
				fprintf(fpo, "PIN, D, I, %s%s\n", io_prefix, nld_pinfo->name) ;
				fprintf(fpo, "PIN, Q, O, NET__%d\n",
					pin_net_num.pickup(nld_pinfo->name)) ;
				fprintf(fpo, "PIN, C, I, NET__%d\n", mc_net) ;
				fprintf(fpo, "END\n") ;
			}
			else {
				fprintf(fpo, "SYM, IBUF__%d, IBUF\n", nld_pinfo->id+1) ;
				fprintf(fpo, "PIN, I, I, %s%s\n", io_prefix, nld_pinfo->name) ;
				fprintf(fpo, "PIN, O, O, NET__%d\n",
					pin_net_num.pickup(nld_pinfo->name)) ;
				fprintf(fpo, "END\n") ;
			}
			}
			else {
				sprintf(str_buf, "pin %s is not converted", nld_pinfo->name) ;
				err_handl(0, str_buf) ;
			}
		}
	}

	// Write Component

	int net_code ;

	forall_plist_items(nld_cinfo, top_module->cmplist) {
		if (cmp_clas_atr.pickup(nld_cinfo->clas) == XNF_CA_HM) {
			if (cmp_name_tbl.iskey(nld_cinfo->clas)) {
				fprintf(fpo, "SYM, %s, %s, DEF=HM\n", \
					nld_cinfo->name, cmp_name_tbl[nld_cinfo->clas]) ;
			}
			else {
				fprintf(fpo, "SYM, %s, %s, DEF=HM\n", \
					nld_cinfo->name, nld_cinfo->clas) ;
			}
		}
		else if (cmp_clas_atr.pickup(nld_cinfo->clas) == XNF_CA_MM) {
			if (cmp_name_tbl.iskey(nld_cinfo->clas)) {
				fprintf(fpo, "SYM, %s, %s, FILE=%s.xnf\n", \
					nld_cinfo->name, cmp_name_tbl[nld_cinfo->clas], \
					cmp_name_tbl[nld_cinfo->clas]) ;
			}
			else {
				fprintf(fpo, "SYM, %s, %s, FILE=%s.xnf\n", \
					nld_cinfo->name, nld_cinfo->clas, nld_cinfo->clas) ;
			}
		}
		else if (cmp_clas_atr.pickup(nld_cinfo->clas) == XNF_CA_FM) {
			if (cmp_name_tbl.iskey(nld_cinfo->clas)) {
				fprintf(fpo, "SYM, %s, %s, FILE=%s.xnf\n", \
					nld_cinfo->name, cmp_name_tbl[nld_cinfo->clas], \
					cmp_name_tbl[nld_cinfo->clas]) ;
			}
			else {
				fprintf(fpo, "SYM, %s, %s, FILE=%s.xnf\n", \
					nld_cinfo->name, nld_cinfo->clas, nld_cinfo->clas) ;
			}
		}
		else {
			if (cmp_name_tbl.iskey(nld_cinfo->clas)) {
				fprintf(fpo, "SYM, %s, %s\n", \
					nld_cinfo->name, cmp_name_tbl[nld_cinfo->clas]) ;
			}
			else {
				fprintf(fpo, "SYM, %s, %s\n", \
					nld_cinfo->name, nld_cinfo->clas) ;
			}
		}
/*
		if (macro_used && !strncmp("HM_", nld_cinfo->clas, 3)) {
			fprintf(fpo, "SYM, %s, %s, DEF=HM\n",
				nld_cinfo->name, (nld_cinfo->clas+3)) ;
		}
		else if (macro_used && !strncmp("hm_", nld_cinfo->clas, 3)) {
			fprintf(fpo, "SYM, %s, %s, DEF=HM\n",
				nld_cinfo->name, (nld_cinfo->clas+3)) ;
		}
		else {
			// Component Name Cross Reference
			if (cmp_name_tbl.iskey(nld_cinfo->clas)) {
				fprintf(fpo, "SYM, %s, %s\n", nld_cinfo->name, cmp_name_tbl[nld_cinfo->clas]) ;
			}
			else {
				fprintf(fpo, "SYM, %s, %s\n", nld_cinfo->name, nld_cinfo->clas) ;
			}
		}
*/
		tmp_npm = mod_clas_tbl.pickup(nld_cinfo->clas) ;
		if (tmp_npm->mtype == IS_PCD_MOD) {
			pmodule = tmp_npm->module.pcd_mod ;
			forall_plist_items(pcd_pinfo, pmodule->pinlist) {
				net_code = net_num_xrf(nld_cinfo->name, pcd_pinfo->name) ;
				if (pcd_pinfo->type == PIN_INPUT) {
					fprintf(fpo, "PIN, %s, I, ", pcd_pinfo->name) ;
					if (net_code == vcc_net) {
						fprintf(fpo, "VCC\n") ;
					} else if (net_code == gnd_net) {
						fprintf(fpo, "GND\n") ;
					} else {
						fprintf(fpo, "NET__%d\n", net_code) ;
					}
				} else if (pcd_pinfo->type == PIN_OUTPUT) {
					fprintf(fpo, "PIN, %s, O, NET__%d\n",
						pcd_pinfo->name, net_code) ;
				} else if (pcd_pinfo->type == PIN_TRISTATE) {
					fprintf(fpo, "PIN, %s, O, NET__%d\n",
						pcd_pinfo->name, net_code) ;
				}
				// xref file making
				if (xrf_file_on) {
					pcd_pin_ref(nld_cinfo->name, pcd_pinfo->name)
						= pcd_pinfo->type ;
				}
			}
		}
		else if (tmp_npm->mtype == IS_NLD_MOD) {
			nmodule = tmp_npm->module.nld_mod ;
			forall_plist_items(nld_pinfo, nmodule->pinlist) {
				net_code = net_num_xrf(nld_cinfo->name, nld_pinfo->name) ;
				if (nld_pinfo->type == P_INPUT
					|| nld_pinfo->type == P_GLOBAL) {
					fprintf(fpo, "PIN, %s, I, ", nld_pinfo->name) ;
					if (net_code == vcc_net) {
						fprintf(fpo, "VCC\n") ;
					} else if (net_code == gnd_net) {
						fprintf(fpo, "GND\n") ;
					} else {
						fprintf(fpo, "NET__%d\n", net_code) ;
					}
				} else if (nld_pinfo->type == P_OUTPUT
					|| nld_pinfo->type == P_TRISTATE) {
					fprintf(fpo, "PIN, %s, O, NET__%d\n",
						nld_pinfo->name, net_code) ;
				}
				// xref file making
				if (xrf_file_on) {
					pcd_pin_ref(nld_cinfo->name, nld_pinfo->name)
						= np_pinconv(nld_pinfo->type) ;
				}
			}
		}
		fprintf(fpo, "END\n") ;
	}

	// Write GSR / GTS STARTUP MODULE

	if (iob_hi_pin) {
		fprintf(fpo, "SYM, GTS_IBUF, IBUF\n") ;
		fprintf(fpo, "PIN, I, I, GTS_iobHI\n") ;
		fprintf(fpo, "PIN, O, O, GTS_IBUF_O\n") ;
		fprintf(fpo, "END\n") ;
	}

	if (use_gsr_on) {
		fprintf(fpo, "SYM, GSR_IBUF, IBUF\n") ;
		fprintf(fpo, "PIN, I, I, p_reset\n")  ;
		fprintf(fpo, "PIN, O, O, GSR_IBUF_O\n") ;
		fprintf(fpo, "END\n") ;
		if (use_gsr_ng) {
			fprintf(fpo, "SYM, GSR_NG_INV, INV\n") ;
			fprintf(fpo, "PIN, I, I, GSR_IBUF_O\n") ;
			fprintf(fpo, "PIN, I, I, GSR_INV_NET\n") ;
			fprintf(fpo, "END\n") ;
		}
		fprintf(fpo, "SYM, STARTUP_M, STARTUP\n") ;
		if (use_gsr_ng) {
			fprintf(fpo, "PIN, GSR, I, GSR_INV_NET\n") ;
		}
		else {
			fprintf(fpo, "PIN, GSR, I, GSR_IBUF_O\n") ;
		}
		if (iob_hi_pin) {
			fprintf(fpo, "PIN, GTS, I, GTS_IBUF_O\n") ;
		}
		if (use_gsr_sc) {
			fprintf(fpo, "PIN, CLK, I, NET__%d\n", mc_net) ;
		}
		fprintf(fpo, "END\n") ;
	}

	// Write Input Pull-Up

	if (pullup_set) {
		i = 1 ;
		forall_plist_items(nld_pinfo, top_module->pinlist) {
			if (!strcmp(nld_pinfo->name, "p_reset")) {
				if (p_reset_on) {
					fprintf(fpo, "SYM, PULLUP__%d, PULLUP\n", i) ;
					fprintf(fpo, "PIN, O, O, p_reset\n") ;
					fprintf(fpo, "END\n") ;
				}
			}
			else if (!strcmp(nld_pinfo->name, "m_clock")) {
				if (!clock_rate) {
					fprintf(fpo, "SYM, PULLUP__%d, PULLUP\n", i) ;
					fprintf(fpo, "PIN, O, O, m_clock\n") ;
					fprintf(fpo, "END\n") ;
				}
			}
			else if (!strcmp(nld_pinfo->name, "s_clock")) {
				if (sbclock_on) {
					fprintf(fpo, "SYM, PULLUP__%d, PULLUP\n", i) ;
					fprintf(fpo, "PIN, O, O, s_clock\n") ;
					fprintf(fpo, "END\n") ;
				}
			}
			else if (!strcmp(nld_pinfo->name, "b_clock")) {
				if (sbclock_on) {
					fprintf(fpo, "SYM, PULLUP__%d, PULLUP\n", i) ;
					fprintf(fpo, "PIN, O, O, b_clock\n") ;
					fprintf(fpo, "END\n") ;
				}
			}
			else if (nld_pinfo->type == P_INPUT 
				|| nld_pinfo->type == P_GLOBAL) {
				fprintf(fpo, "SYM, PULLUP__%d, PULLUP\n", i) ;
				fprintf(fpo, "PIN, O, O, %s%s\n", io_prefix, nld_pinfo->name) ;
				fprintf(fpo, "END\n") ;
			}
			i++ ;
		}
	}

	// Write I/O Pads

	forall_plist_items(nld_pinfo, top_module->pinlist) {
		if (nld_pinfo->type == P_INPUT || nld_pinfo->type == P_GLOBAL) {
			if (!strcmp(nld_pinfo->name, "p_reset")) {
				if (p_reset_on || use_gsr_on) 
					fprintf(fpo, "EXT, p_reset, I\n") ;
			}
			else if (!strcmp(nld_pinfo->name, "m_clock")) {
				if (!clock_rate) 
					fprintf(fpo, "EXT, m_clock, I\n") ;
			}
			else if (!strcmp(nld_pinfo->name, "s_clock")) { 
				if (sbclock_on)
					fprintf(fpo, "EXT, s_clock, I\n") ;
			}
			else if (!strcmp(nld_pinfo->name, "b_clock")) {
				if (sbclock_on)
					fprintf(fpo, "EXT, b_clock, I\n") ;
			}
			else {
				fprintf(fpo, "EXT, %s%s, I\n", io_prefix, nld_pinfo->name) ;
			}
		} else if (nld_pinfo->type == P_OUTPUT) {
			fprintf(fpo, "EXT, %s%s, O\n", io_prefix, nld_pinfo->name) ;
		}
	}
	if (iob_hi_pin) {
		fprintf(fpo, "EXT, GTS_iobHI, I\n") ;
	}

	fprintf(fpo, "PWR, 0, GND\n") ;
	fprintf(fpo, "PWR, 1, VCC\n") ;
	fprintf(fpo, "EOF\n") ;

	fclose(fpo) ;

	// Write XRef File

	NLDPinType	tmp_npin_type ;
	PCDPinType	tmp_ppin_type ;

	if (xrf_file_on) {
		fprintf(fpx, "; Net Name Cross Reference of Design : %s\n", top_mod_file) ;
		forall_plist_items(nld_pinfo, top_module->pinlist) {
			top_mod_pins.entry(nld_pinfo->name, nld_pinfo->type) ;
		}
		forall_plist_items(nld_ninfo, top_module->netlist) {
			forall_list_items(nld_nitem, nld_ninfo->nlink) {
				if (nld_nitem->etyp == IS_PIN) {
					tmp_npin_type = top_mod_pins.pickup(nld_nitem->signal) ;
					if (tmp_npin_type == P_INPUT
						||	tmp_npin_type == P_INPUT_CONNECTED
						||	tmp_npin_type == P_GLOBAL) {
						fprintf(fpx, "NET__%d\t<P> %s\n",
							nld_ninfo->id+1, nld_nitem->signal) ;
					}
				}
				else if (nld_nitem->etyp == IS_CMP) {
					tmp_ppin_type 
						= pcd_pin_ref(nld_nitem->signal, nld_nitem->sigpin) ;
					if (tmp_ppin_type == PIN_OUTPUT
						|| tmp_ppin_type == PIN_TRISTATE) {
						fprintf(fpx, "NET__%d\t<C> %s\n",
							nld_ninfo->id+1, nld_nitem->signal) ;
					}
				}
			}
		}
		fclose(fpx) ;
	}

}
