//
// nld_xnf definitions and functions header
//

#ifndef _nld_xnf_h_
#define _nld_xnf_h_

#define XNF_CA_HM 1
#define XNF_CA_SM 2
#define XNF_CA_MM 3
#define XNF_CA_FM 4
#define XNF_CA_CP 0
/*
#define IS_PCD_MOD 1
#define IS_NLD_MOD 1
*/
typedef AssocArray<char*> Xrf_Tbl ;

inline void
sw_bracket(char *str)
{
	for (int i=0; str[i] != '\0'; i++) {
		switch (str[i]) {
		case '[' :
			str[i] = '<' ;
			break ;
		case ']' :
			str[i] = '>' ;
			break ;
		default :
			;
		}
	}
}

inline
PCDPinType
np_pinconv (NLDPinType npt) {

	switch (npt) {
	case P_INPUT :
		return PIN_INPUT ;
	case P_INPUT_CONNECTED :
		return PIN_INPUT ;
	case P_OUTPUT :
		return PIN_OUTPUT ;
	case P_OUTPUT_CONNECTED :
		return PIN_OUTPUT ;
	case P_BIDIRECT :
		return PIN_BIDIRECT ;
	case P_BIDIRECT_CONNECTED :
		return PIN_BIDIRECT ;
	case P_TRISTATE :
		return PIN_TRISTATE ;
	case P_TRISTATE_CONNECTED :
		return PIN_TRISTATE ;
	case P_CONNECTED :
		return PIN_UNDEF ;
	case P_GLOBAL :
		return PIN_INPUT ;
	case P_UNUSED :
		return PIN_UNUSED ;
	case P_UNUSED_CONNECTED :
		return PIN_UNUSED ;
	case P_UNDEFINED :
		return PIN_UNDEF ;
	default :
		;
	}
	return PIN_UNDEF ;
}

int
decode_spf(PCDCmdList* PCL, char* pname, Xrf_Tbl& XTB)
{
	FILE* fp ;
	PCDStrList* psl ;
	char* str ;
	char* tmps ;
	char  mfname[256] ;
	char  cname[256] ;
	int   sflag ;
	int   nflag = 0 ;
	int   mflag = 0 ;
	int   rflag = 0 ;

	int   col   = 0 ;
	int   mod   = 0 ;

	sflag = XNF_CA_CP ;

	mod = 0 ;
	forall_plist_items(psl, PCL) {
		col = 1 ;
		mod = 0 ;
		forall_plist_items(str, psl) {
			if (!strcmp(str, "xnf-clas") && (col == 1)) {
				mod = 1 ;
				col++ ;
			}
			else if (!strcmp(str, "xnf-name") && (col == 1)) {
				mod = 2 ;
				col++ ;
			}
			else if (!strcmp(str, "xnf-memdef") && (col == 1)) {
				mod = 3 ;
				col++ ;
			}
			else if ((mod == 1) && (col == 2)) {
				if        (!strcmp(str, "HARD_MACRO")) {
					sflag = XNF_CA_HM ;
				} else if (!strcmp(str, "SOFT_MACRO")) {
					sflag = XNF_CA_SM ;
				} else if (!strcmp(str, "MEM_MODULE")) {
					sflag = XNF_CA_MM ;
				} else if (!strcmp(str, "PRIMITIVE"))  {
					sflag = XNF_CA_CP ;
				} else if (!strcmp(str, "MACRO_FILE"))  {
					sflag = XNF_CA_FM ;
				} else {
					cerr << "Error : xnf-clas is not defined in " << pname ;
					cerr << ".pcd" << endl ;
					return (-1) ;
				}
				rflag = 1 ;
				col++ ;
			}
			else if ((mod == 2) && (col == 2)) {
				tmps = new char[strlen(str)+1] ;
				strcpy(tmps, str) ;
				strcpy(cname, str) ;
				XTB[pname] = tmps ;
				nflag = 1 ;
				rflag = 1 ;
				col++ ;
			}
			else if (mod == 3) {
				if (!mflag) {
					if (nflag) {
						sprintf(mfname, "%s.mem", cname) ;
					} else {
						sprintf(mfname, "%s.mem", pname) ;
					}
					if ((fp = fopen(mfname, "wt")) == NULL) {
						cerr << "Error : cannot create mem file" << endl ;
						return (-1) ;
					}
					cerr << "nld_xnf : write memory definition file " ;
					cerr << mfname << endl ;
					fprintf(fp, "%s ", str) ;
					mflag = 1 ;
					rflag = 1 ;
				}
				else {
					fprintf(fp, "%s ", str) ;
				}
				col++ ;
			}
			else {
				cerr << "Error : Illegal def-spec field in " << pname ;
				cerr << ".pcd" << endl ;
				return (-1) ;
			}
		}
		if (mflag) {
			fprintf(fp, "\n") ;
		}
	}

	if (mflag) {
		fclose(fp) ;
	}

	if (rflag) return sflag ;
	else {
		cerr << "Error : xnf command is not defined in " << pname ;
		cerr << ".pcd" << endl ;
		return (-1) ;
	}
}

int decode_spf(PCDCmdList*, char*, Xrf_Tbl&) ;

#endif
