/*
	DATA TYPE DEFINITION OF EXPANDED NLD FILE
    Written and Revised by Akihiro Tsutsui (NTT Optical Network Systems Labs.)
    Copyright (c) 1994, 1995 Nippon Telegraph & Telephone Corporation
*/

#ifndef	_nlddefs_h_
#define _nlddefs_h_ 1

#include <string.h>

#include "ulist.h"
#include "pcddefs.h"

#define NLD_CMP_PIN 0
#define NLD_CMP_CMP 1

#define IS_NLD_MOD 1
#define IS_PCD_MOD 2


typedef enum {
	P_NULL = 0,
	P_INPUT,
	P_INPUT_CONNECTED,
	P_OUTPUT,
	P_OUTPUT_CONNECTED,
	P_BIDIRECT,
	P_BIDIRECT_CONNECTED,
	P_TRISTATE,
	P_TRISTATE_CONNECTED,
	P_CONNECTED,
	P_GLOBAL,
	P_UNUSED,
	P_UNUSED_CONNECTED,
	P_UNDEFINED
} NLDPinType ;

typedef enum {
	PN_UNDEF,
	PN_CLOCK,
	PN_TRISTATE,
	PN_STATIC
} NLDPinNote ;

typedef enum {
	IS_PIN,
	IS_CMP,
	IS_UDF
} NLDElmType ;

typedef enum {
	IS_NLD_CMP,
	IS_PCD_CMP,
	IS_PIN_CMP,
	IS_UNKNOWN
} NLDCmpType ;

class NLDPinInfo {
	public:
	int			id ;
	char		*name ;
	NLDPinType	type ;
	NLDPinNote	note ;
	NLDCmpType  cmp_type ;
	// constructor & destructor
	NLDPinInfo()
	{
		id = -1; name = 0; type = P_UNDEFINED; note = PN_UNDEF;
		cmp_type = IS_PIN_CMP ;
	}
	NLDPinInfo(int n, const char* pn, NLDPinType pt, NLDPinNote nt = PN_UNDEF)
	{
		id = n ;
		name = new char[strlen(pn)+1] ;
		strcpy(name, pn) ;
		type = pt ;
		note = nt ;
		cmp_type = IS_PIN_CMP ;
	}
	~NLDPinInfo()
	{
		delete name ;
	}
} ;

class NLDCmpInfo {
	public:
	int		id ;
	char	*name ;
	char	*clas ;
	NLDCmpType type ;
	// constructor & destructor
	NLDCmpInfo()
	{
		id = -1; name = 0; clas = 0; type = IS_UNKNOWN ;
	}
	NLDCmpInfo(int n, const char* cls, const char* ist)
	{
		id = n ;
		name = new char[strlen(ist)+1] ;
		strcpy(name, ist) ;
		clas = new char[strlen(cls)+1] ;
		strcpy(clas, cls) ;
		type = IS_UNKNOWN ;
	}
	~NLDCmpInfo()
	{
		delete name ;
		delete clas ;
	}
} ;

class NLDNetItem {
	public:
	char		*signal ;
	char		*sigpin ;
	NLDElmType	etyp ;
	PCDPinType	atrb ;
	// constructor & destructor
	NLDNetItem()
	{
		signal = 0; sigpin = 0; etyp = IS_UDF; atrb = PIN_UNDEF;
	}
	NLDNetItem(const char* signame)
	{
		char	chtest[256] ;
		char*	ext = 0 ;
		strcpy(chtest, signame) ;

		for (int i=0; chtest[i] != '\0'; i++) {
			if (chtest[i] == '.') {
				chtest[i] = '\0' ;
				ext = (chtest+i+1) ;
			}
		}

		signal = new char[strlen(chtest)+1] ;
		strcpy(signal, chtest) ;
		if (ext) {
			sigpin = new char[strlen(ext)+1] ;
			strcpy(sigpin, ext) ;
			etyp = IS_CMP ;
		}
		else {
			sigpin = 0 ;
			etyp = IS_PIN ;
		}
//		etyp = IS_UDF ;
		atrb = PIN_UNDEF ;
	}
	~NLDNetItem()
	{
		delete signal ;
		delete sigpin ;
	}
} ;
typedef ulist<NLDNetItem*> NLDNetLink ;

class NLDNetInfo {
	public:
	int		id ;
	NLDNetLink	nlink ;
	// constructor & destructor
	NLDNetInfo()
	{
		id = -1;
	}
	NLDNetInfo(int n)
	{
		id = n ;
	}
	~NLDNetInfo()
	{
		nlink.clear() ;
	}
} ;

typedef ulist<char*>       NLDStrList ;
typedef ulist<NLDStrList*> NLDCmdList ;
typedef	ulist<NLDPinInfo*> NLDPinList ;
typedef ulist<NLDCmpInfo*> NLDCmpList ;
typedef ulist<NLDNetInfo*> NLDNetList ;

class NLD_MODULE {
	public :
	int		id ;
	char*	name ;
	NLDPinList*	pinlist ;
	NLDCmpList*	cmplist ;
	NLDNetList*	netlist ;
	NLDCmdList*	defspec ;

//  misc entry is reserved for future use
//	NLDCmdList*	defmisc ;

	// constructor & destructor
	NLD_MODULE()
	{
		id = 0 ;
		name = 0 ;
		pinlist = 0 ;
		cmplist = 0 ;
		netlist = 0 ;
		defspec = 0 ;
	}
	NLD_MODULE(int n)
	{
		id = n ;
		name = 0 ;
		pinlist = 0 ;
		cmplist = 0 ;
		netlist = 0 ;
		defspec = 0 ;
	}
	NLD_MODULE(int n, const char* str)
	{
		id = n ;
		name = new char [strlen(str)+1] ;
		strcpy(name, str) ;
		pinlist = 0 ;
		cmplist = 0 ;
		netlist = 0 ;
		defspec = 0 ;
	}
	~NLD_MODULE() {}

} ;

typedef ulist<NLD_MODULE*> NLD_MODULE_LIST ;

typedef union {
	NLD_MODULE* nld_mod ;
	PCD_MODULE* pcd_mod ;
} NLD_PCD_PTR ;

class NLD_PCD_MODULE {
	public :
	int			mtype  ;
	NLD_PCD_PTR	module ;

	NLD_PCD_MODULE()
	{
		mtype  = 0 ;
	}
	NLD_PCD_MODULE(int mt, NLD_PCD_PTR mpt)
	{
		mtype  = mt ;
		module = mpt ;
	}
	NLD_PCD_MODULE(NLD_MODULE* mpt)
	{
		mtype           = IS_NLD_MOD ;
		module.nld_mod = mpt ;
	}
	NLD_PCD_MODULE(PCD_MODULE* mpt)
	{
		mtype           = IS_PCD_MOD ;
		module.pcd_mod = mpt ;
	}
	~NLD_PCD_MODULE() {}
} ;

#endif
