/*
  lex file for NLD format
  Written and Revised by Akihiro Tsutsui (NTT Optical Network Systems Labs.)
  Copyright (c) 1994, 1995 Nippon Telegraph & Telephone Corporation
*/

%{

#include "nlddefs.h"
#include "nldparse.h"
#include "nldyacc.h"

%}

blank	[ \n\t]
alpha	[A-Za-z]
idchars	[-A-Za-z0-9\[\]_<>\.]
alnum	[-A-Za-z0-9]
string	[^ \n\t\']

%start USR SPECIAL

%%
<INITIAL>{alpha}{idchars}* {
			    if (strcmp(yytext, "def-module") == 0) {
				return DEF_MODULE;
			    } else if (strcmp(yytext, "def-pin") == 0) {
				return DEF_PIN;
			    } else if (strcmp(yytext, "def-comp") == 0) {
				return DEF_COMP;
			    } else if (strcmp(yytext, "def-net") == 0) {
				return DEF_NET;
			    } else if (strcmp(yytext, "def-special") == 0) {
				BEGIN USR ;
				return DEF_SPECIAL;
			    } else if (strcmp(yytext, "type") == 0) {
				return TYPE;
			    } else if (strcmp(yytext, "note") == 0) {
				return NOTE;
			    } else if (strcmp(yytext, "input-connected") == 0) {
				return INPUT_CONNECTED;
			    } else if (strcmp(yytext, "input") == 0) {
				return INPUT;
			    } else if (strcmp(yytext, "output-connected") == 0) {
				return OUTPUT_CONNECTED;
			    } else if (strcmp(yytext, "output") == 0) {
				return OUTPUT;
			    } else if (strcmp(yytext, "bidirect-connected") == 0) {
				return BIDIRECT_CONNECTED;
			    } else if (strcmp(yytext, "bidirect") == 0) {
				return BIDIRECT;
			    } else if (strcmp(yytext, "tristate-connected") == 0) {
				return TRISTATE_CONNECTED;
			    } else if (strcmp(yytext, "tristate") == 0) {
				return TRISTATE;
			    } else if (strcmp(yytext, "connected") == 0) {
				return CONNECTED;
			    } else if (strcmp(yytext, "global") == 0) {
				return GLOBAL;
			    } else if (strcmp(yytext, "unused-connected") == 0) {
				return UNUSED_CONNECTED;
			    } else if (strcmp(yytext, "unused") == 0) {
				return UNUSED;
			    } else if (strcmp(yytext, "undefined") == 0) {
				return UNDEFINED;
			    } else if (strcmp(yytext, "clock") == 0) {
				return CLOCK;
			    } else if (strcmp(yytext, "static") == 0) {
				return STATIC;

				/* This version prohibits using user defined def-X */
				/*
			    } else if (strncmp("def-", yytext, 4) == 0) {
				return DEF_MISC;
				*/

			    } else {
				return IDENTIFIER; 
			    }
			}

<INITIAL>"("			{ return LPAREN; }
<INITIAL>")"			{ return RPAREN; }


<INITIAL>{blank}+	;	  /* white space */
<INITIAL>";"[^\n]* 	;	  /* comment line */
<INITIAL>.			{ 
					printf ("%s\n", yytext) ;
					perror("state <INITIAL> bad character");
					}

<USR>")"	{ BEGIN INITIAL; return RPAREN; }
<USR>"\'"	{ BEGIN SPECIAL; }
<USR>{blank}+	;	  /* white space */
<USR>{alpha}{idchars}* { return IDENTIFIER; }
<USR>.		{
			printf ("%s\n", yytext) ;
			perror("state <USR> bad character");
			}

<SPECIAL>"\'"	{ BEGIN USR; }
<SPECIAL>{blank}+	;	/* white space in def-special */
<SPECIAL>{string}+	{ return IDENTIFIER; }

%%
