/*
  yacc file for NLD format
  Written and Revised by Akihiro Tsutsui (NTT Optical Network Systems Labs.)
  Copyright (c) 1994, 1995 Nippon Telegraph & Telephone Corporation

*/

%{


#include <stdio.h>
#include <string.h>
#include <alloca.h>
#include <iostream.h>
#include "nlddefs.h"
#include "ulist.h"
#include "nldparse.h"

#define YYDEBUG 1

extern char *yytext ;
extern int  yylex() ;
extern void yyerror(char*) ;

extern char nld_tmp_str[256] ;

static NLDPinList* nld_pin_list ;
static NLDCmpList* nld_cmp_list ;
static NLDNetList* nld_net_list ;
static NLDCmdList* nld_spc_list ;
static NLD_MODULE* nld_module_p ;

extern NLD_MODULE_LIST* nld_modules ;
extern int nld_mod_count ;

static int pin_id = 0 ;
static int cmp_id = 0 ;
static int net_id = 0 ;
static int spc_id = 0 ;

static NLDPinInfo* nld_tmp_pinfo ;
static NLDCmpInfo* nld_tmp_cinfo ;
static NLDNetInfo* nld_tmp_ninfo ;
static NLDNetItem* nld_tmp_nitem ;
static NLDStrList* nld_tmp_slist ;

static char* nld_str_dupl ;

%}

%union {
	char *strval ;
	int  ival    ;
	NLDPinType npin ;
}

%type <strval> identifier
%type <npin>   pin_type
%type <npin>   ptype

%token LPAREN RPAREN

%token DEF_MODULE
%token DEF_PIN
%token DEF_COMP
%token DEF_NET
%token DEF_SPECIAL

%token TYPE
%token INPUT
%token INPUT_CONNECTED
%token OUTPUT
%token OUTPUT_CONNECTED
%token BIDIRECT
%token BIDIRECT_CONNECTED
%token TRISTATE
%token TRISTATE_CONNECTED
%token CONNECTED
%token GLOBAL
%token UNUSED
%token UNUSED_CONNECTED
%token UNDEFINED

%token NOTE
%token DOT
%token STATIC
%token CLOCK

%token IDENTIFIER

%start nlds

%%

nlds		:		module_defs
			;

module_defs	:		/* empty */
			|		module_defs module_def
			;

module_def	:		LPAREN DEF_MODULE identifier
					{
					nld_module_p = new NLD_MODULE(nld_mod_count++, $3) ;
					nld_pin_list = new NLDPinList ;
					nld_cmp_list = new NLDCmpList ;
					nld_net_list = new NLDNetList ;
					nld_spc_list = new NLDCmdList ;
					nld_module_p->pinlist = nld_pin_list ;
					nld_module_p->cmplist = nld_cmp_list ;
					nld_module_p->netlist = nld_net_list ;
					nld_module_p->defspec = nld_spc_list ;
					nld_modules->append(nld_module_p) ;
					}
					defs RPAREN
			;

defs		:		/* empty */
			|		defs def
			;

def			:		pin_def
			|		comp_def
			|		net_def
			|		spc_def
			;

pin_def		:		LPAREN DEF_PIN identifier 
					{
					strcpy(nld_tmp_str, $3) ;
					}
					pin_type pin_note RPAREN
			;

pin_type	:		/* empty */
					{
					$$ = P_UNDEFINED ;
					nld_tmp_pinfo =
						new NLDPinInfo(pin_id++, nld_tmp_str, P_UNDEFINED) ;
					nld_pin_list->append(nld_tmp_pinfo) ;
					}
			|		TYPE ptype
					{
					$$ = $2 ;
					nld_tmp_pinfo =
						new NLDPinInfo(pin_id++, nld_tmp_str, $2) ;
					nld_pin_list->append(nld_tmp_pinfo) ;
					}
			;

ptype       :       INPUT
					{ $$ = P_INPUT ;}
            |       INPUT_CONNECTED
					{ $$ = P_INPUT_CONNECTED ;}
            |       OUTPUT
					{ $$ = P_OUTPUT ;}
            |       OUTPUT_CONNECTED
					{ $$ = P_OUTPUT_CONNECTED ;}
            |       BIDIRECT
					{ $$ = P_BIDIRECT ;}
            |       BIDIRECT_CONNECTED
					{ $$ = P_BIDIRECT_CONNECTED ;}
            |       TRISTATE
					{ $$ = P_TRISTATE ;}
            |       TRISTATE_CONNECTED
					{ $$ = P_TRISTATE_CONNECTED ;}
            |       CONNECTED
					{ $$ = P_CONNECTED ;}
            |       GLOBAL
					{ $$ = P_GLOBAL ;}
            |       UNUSED
					{ $$ = P_UNUSED ;}
            |       UNUSED_CONNECTED
					{ $$ = P_UNUSED_CONNECTED ;}
            |       UNDEFINED
					{ $$ = P_UNDEFINED ;}
            ;

pin_note    :       /* empty */
            |       NOTE notice
            ;

notice      :       CLOCK
            |       TRISTATE
            |       STATIC
            ;

comp_def    :       LPAREN DEF_COMP identifier 
					{
					strcpy(nld_tmp_str, $3) ;
					}
					compinsts RPAREN
            ;
compinsts	:		compinst
			|		compinsts compinst
			;

compinst	:		identifier
					{
					nld_tmp_cinfo =
						new NLDCmpInfo(cmp_id++, nld_tmp_str, $1) ;
					nld_cmp_list->append(nld_tmp_cinfo) ;
					}
			;

net_def     :       LPAREN DEF_NET
					{ 
					nld_tmp_ninfo = new NLDNetInfo(net_id++) ;
					nld_net_list->append(nld_tmp_ninfo) ;
					}
					signals RPAREN
			;

signals		:       /* empty */
            |       signals signal
            ;

signal      :       identifier 
					{
					nld_tmp_nitem = new NLDNetItem($1) ;
					nld_tmp_ninfo->nlink.append(nld_tmp_nitem) ;
					}
            ;

spc_def		:		LPAREN DEF_SPECIAL
					{
					nld_tmp_slist = new NLDStrList ;
					nld_spc_list->append(nld_tmp_slist) ;
					spc_id++;
					}
					sp_atoms RPAREN
			;

sp_atoms	:		/* empty */
			|		sp_atoms sp_atom
			;

sp_atom		:		identifier
					{
					nld_str_dupl = new char[strlen($1)+1] ;
					strcpy(nld_str_dupl, $1) ;
					nld_tmp_slist->append(nld_str_dupl) ;
					}
			;

identifier  :       IDENTIFIER
					{
					$$ = yytext ;
					}
	    	;

%%

char nld_tmp_str[256] ;

NLD_MODULE_LIST* nld_modules ;
int nld_mod_count ;

int
parse_nldx(FILE *fp, NLD_MODULE_LIST& NML)
{
	// nld parsing global variables

	extern NLD_MODULE_LIST* nld_modules ;
	extern int nld_mod_count ;

#ifdef PARSER_DEBUG
	extern int yydebug ;
#endif

	// nld parsing operation variables

	void yyrestart(FILE *f) ;
	extern int yyparse() ;
	extern FILE* yyin ;
	int retval ;

	static int parseflg = 0 ;
	nld_mod_count = 0 ;
	nld_modules = &NML ;

#ifdef PARSER_DEBUG
	yydebug = 1 ;
#endif

	if (parseflg) {
		yyrestart(fp) ;
		retval = yyparse() ;
	}
	else {
		yyin = fp ;
		retval = yyparse() ;
		parseflg = 1 ;
	}

	if (retval) {
		perror("parse error") ;
		return (-1) ;
	}
	
	return nld_mod_count ;
}
