/*
	DATA TYPE DEFINITION FOR PCD FILE

    Written and Revised by Akihiro Tsutsui (NTT Optical Network Systems Labs.)
    Copyright (c) 1994 Nippon Telegraph & Telephone Corporation
*/

#ifndef _pcddef_h_
#define _pcddef_h_ 1

#include <string.h>
#include "ulist.h"

typedef enum {
	PIN_NULL = 0,
	PIN_INPUT,
	PIN_OUTPUT,
	PIN_BIDIRECT,
	PIN_TRISTATE,
	PIN_UNUSED,
	PIN_UNDEF
} PCDPinType ;

class PCDPinInfo {
	public:
	int			id ;
	char		*name ;
	PCDPinType	type ;
	// constructor & destructor
	PCDPinInfo()
	{
		id = -1; name = 0; type = PIN_UNDEF;
	}
	PCDPinInfo(int n, char* pn, PCDPinType pt)
	{
		id = n ;
		name = new char [strlen(pn)+1] ; ;
		strcpy(name, pn) ;
		type = pt ;
	}
	PCDPinInfo(PCDPinInfo& pinfo_def)
	{
		id = pinfo_def.id ;
		name = new char [strlen(pinfo_def.name)+1] ;
		strcpy(name, pinfo_def.name) ;
		type = pinfo_def.type ;
	}
	~PCDPinInfo()
	{
		delete name ;
	}
	// self cleaning
	void clear()
	{
		id = -1 ;
		name = 0 ;
		type = PIN_UNDEF ;
	}
} ;

/*
	class PCDLogicInfo {
	} ;
*/

typedef ulist<char*>		PCDStrList ;
typedef ulist<PCDStrList*>	PCDCmdList ;
typedef ulist<PCDPinInfo*>	PCDPinList ;

/*
	typedef ulist<PCDLogicInfo*> PCDLogicList ;
*/

class PCD_MODULE {
	public :
	int			id ;
	char*		name ;
	char*		impl_trg ;
	char*		impl_mth ;
	PCDPinList*	pinlist ;
	PCDCmdList* defspec ;
	// constructor & destructor
	PCD_MODULE()
	{
		id = 0 ;
		name = 0 ;
		pinlist = 0 ;
		impl_trg = 0 ;
		impl_mth = 0 ;
		defspec = 0 ;
	}
	PCD_MODULE(int n)
	{
		id = n ;
		name = 0 ;
		impl_trg = 0 ;
		impl_mth = 0 ;
		pinlist = 0 ;
		defspec = 0 ;
	}
	PCD_MODULE(int n, const char* str)
	{
		id = n ;
		name = new char [strlen(str)+1] ; ;
		strcpy(name, str) ;
		impl_trg = 0 ;
		impl_mth = 0 ;
		pinlist = 0 ;
		defspec = 0 ;
	}
	~PCD_MODULE() {}
} ;

typedef ulist<PCD_MODULE*> PCD_MODULE_LIST ;

#endif
