/*
  lex file for pcd format

  Written and Revised by Akihiro Tsutsui (NTT Optical Network Systems Labs.)
  Copyright (c) 1994, 1995 Nippon Telegraph & Telephone Corporation
*/

%{

#include "pcddefs.h"
#include "pcdparse.h"
#include "pcdyacc.h"

int pcdparse_lineno ;

%}

blank		[ \t]
alpha		[A-Za-z]
idchars		[-A-Za-z0-9\[\]_<>\.]
alnum		[-A-Za-z0-9_]
mant1		([0-9]+\.?[0-9]*) 
mant2		([0-9]*\.[0-9]+)
exp			([eE][-+]?[0-9]+)
float		(\-?(([0-9]+\.?[0-9]*)|([0-9]*\.[0-9]+))([eE][-+]?[0-9]+)?)
int			((\-?[1-9]+[0-9]*)|0)
operator	[\-\+\*\%\/\\~\?]

%%
<INITIAL>{int}			{ return INTEGER; }
<INITIAL>{float}		{ return FLOAT; }
<INITIAL>{alpha}{idchars}* {
			    if (strcmp(yytext, "def-module") == 0) {
				return DEF_MODULE;
			    } else if (strcmp(yytext, "def-pin") == 0) {
				return DEF_PIN;
			    } else if (strcmp(yytext, "def-delay") == 0) {
				return DEF_DELAY;
				} else if (strcmp(yytext, "def-symbol") == 0) {
				return DEF_SYMBOL;
				} else if (strcmp(yytext, "def-constraint") == 0) {
				return DEF_CONSTRAINT;
				} else if (strcmp(yytext, "def-function") == 0) {
				return DEF_FUNCTION;
				} else if (strcmp(yytext, "def-logic") == 0) {
				return DEF_LOGIC;

				} else if (strcmp(yytext, "def-special") == 0) {
				return DEF_SPECIAL;

				} else if (strcmp(yytext, "power") == 0) {
				return POWER;
				} else if (strcmp(yytext, "area") == 0) {
				return AREA;
				} else if (strcmp(yytext, "gates") == 0) {
				return GATES;
				} else if (strcmp(yytext, "cell_name") == 0) {
				return CELL_NAME;

				} else if (strcmp(yytext, "load") == 0) {
				return LOAD;
			    } else if (strcmp(yytext, "note") == 0) {
				return NOTE;
			    } else if (strcmp(yytext, "type") == 0) {
				return TYPE;

			    } else if (strcmp(yytext, "input") == 0) {
				return INPUT;
			    } else if (strcmp(yytext, "output") == 0) {
				return OUTPUT;
			    } else if (strcmp(yytext, "bidirect") == 0) {
				return BIDIRECT;
			    } else if (strcmp(yytext, "tristate") == 0) {
				return TRISTATE;
			    } else if (strcmp(yytext, "unused") == 0) {
				return UNUSED;

			    } else if (strcmp(yytext, "file") == 0) {
				return F_NAME;
/*
			    } else if (strcmp(yytext, "free_reg") == 0) {
				return FREG_OP;
			    } else if (strcmp(yytext, "and") == 0) {
				return AND_OP;
			    } else if (strcmp(yytext, "or") == 0) {
				return OR_OP;
*/
			    } else {
				return IDENTIFIER; 
			    }
			}

<INITIAL>"("	{ return LPAREN; }
<INITIAL>")"	{ return RPAREN; }

<INITIAL>{operator}		{ return yytext[0] ;}
<INITIAL>"\^"			{ return NOT ; }
<INITIAL>{blank}+		;  /* white space */
<INITIAL>"\n"			{ pcdparse_lineno++; }
<INITIAL>";!"			;  /* white space (special feature) */
<INITIAL>";"[^!][^\n]*		;  /* comment line */
<INITIAL>.			{ 
			printf ("%s\n", yytext) ;
			perror("bad character") ;
			}
%%
