//
// nld_xnf message printout functions
//

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>

void print_usage()
{

  cerr << endl ;
  cerr << "usage: nld_xnf [-d <part_type>][-s <speed>][-e <pin_prefix>][-i <clock>]" << endl ;
//  cerr << "               [-o <output_file>][-x <xrf_file>][-p][-f][-r][-b][-t][-x]" << endl ;
  cerr << "               [-o <output_file>][-x <xrf_file>][-l <FF insert side>]" << endl ;
  cerr << "               [-p][-f][-r][-R[+]][-RS[+]][-b][-t][-x][-F][-Z]" << endl ;
//  cerr << "               [-m][-l][-z]" << endl ;
  cerr << "               topmodule.nld  nld/pcdfiles.dirs .." << endl ;
  cerr << "   options :" << endl ;
  cerr << "             -d <part_type> : select target device" << endl ;
  cerr << "                              (default = 3040PG132)" << endl ;
  cerr << "             -s <speed>     : select device speed" << endl ;
  cerr << "                              speed = 70, 100*, 125, 150 (xc3000)" << endl ;
  cerr << "                              speed = 3*, 4, 5           (xc3100)" << endl ;
  cerr << "                              speed = 2, 5*, 6           (xc4000)" << endl ;
  cerr << "                              * means default speed for each device" << endl ;
  cerr << "             -e <pin_prefix>: set io_pin name prefix" << endl ;
//  cerr << "             -c <cmd_file>  : use command file" << endl ;
//cerr << "                              set io_pin name prefix" << endl ;
//cerr << "                              default prefix = x_" << endl ;
  cerr << "             -i <clock>     : direct connect m_clock to internal OSC" << endl ;
  cerr << "                              clock = 15 | 15k | 500k | 8m" << endl ;
  cerr << "                              (for xc4000 series only)" << endl ;
  cerr << "             -o <out_file>  : set output file name" << endl ;
  cerr << "                              default output to stdout" << endl ;
  cerr << "             -x <xrf_file>  : net cross-reference file name" << endl ;
  cerr << "             -l {I|O|B}     : insert i/o flip-flop" << endl ;
  cerr << "                              Input, Output, Both Pins" << endl ;
  cerr << "             -p             : pull-up all input pin" << endl ;
  cerr << "             -f             : output slew rate control set to fast" << endl ;
  cerr << "                              (for xc4000 series only)" << endl ;
  cerr << "             -r             : enable p_reset net" << endl ;
  cerr << "             -R[+]          : connect p_reset LCA's global reset" << endl ;
  cerr << "                              p_reset logic default is negative" << endl ;
  cerr << "             -RS[+]         : reset is synchronized with m_clock" << endl ;
  cerr << "                              (for xc4000 series only)" << endl ;
  cerr << "             -b             : enable s_clock, b_clock net" << endl ;
  cerr << "             -t             : add readback & JTAG net" << endl ;
  cerr << "                              (for xc4000 series only)" << endl ;
//  cerr << "             -m             : use hard macros" << endl ;
// cerr << "                              (for xc4000 series only)" << endl ;
 cerr << "             -F             : if possible, flush registers and use io_dff" << endl ;
//  cerr << "             -n             : preserve component names" << endl ;
//  cerr << "             -z             : network optimization (yet)" << endl ;
  cerr << "             -Z             : generate GTS_iobHI pin" << endl ;
  cerr << "                              (for xc4000 series only)" << endl ;
//  cerr << "             -?             : print this usage" << endl ;
//  cerr << "             (x)            : not implemented yet ^^;" << endl ;
}

void
write_jt(FILE *fp, int reset_on)
{

	// Startup

    fprintf (fp, "SYM, PAD__ST, PAD\n") ;
    fprintf (fp, "PIN, O, I, G_START\n") ;
    fprintf (fp, "END\n") ;
    fprintf (fp, "SYM, PULLUP__ST, PULLUP\n") ;
    fprintf (fp, "PIN, O,    O, G_START\n") ;
    fprintf (fp, "END\n") ;
    if(reset_on) {
	fprintf (fp, "SYM, IBUF__ST, IBUF\n") ;
    } else {
	fprintf (fp, "SYM, BUFGP__ST, BUFGP\n") ;
    }
    fprintf (fp, "PIN, I,    I, G_START\n") ;
    fprintf (fp, "PIN, O,    O, LN__ST1\n") ;
    fprintf (fp, "END\n") ;
    fprintf (fp, "SYM, INV__ST, INV\n") ;
    fprintf (fp, "PIN, I,    I, LN__ST1\n") ;
    fprintf (fp, "PIN, O,    O, LN__ST2\n") ;
    fprintf (fp, "END\n") ;
    fprintf (fp, "SYM, STARTUP__ST, STARTUP\n") ;
    fprintf (fp, "PIN, GSR,  I, LN__ST2\n") ;
    fprintf (fp, "END\n") ;

	// Readback

    fprintf (fp, "SYM, MD0__RD, MD0\n") ;
    fprintf (fp, "PIN, I,    O, LN__RD1\n") ;
    fprintf (fp, "END\n") ;
    fprintf (fp, "SYM, IBUF__RD, IBUF\n") ;
    fprintf (fp, "PIN, I,    I, LN__RD1\n") ;
    fprintf (fp, "PIN, O,    O, LN__RD2\n") ;
    fprintf (fp, "END\n") ;
    fprintf (fp, "SYM, RDBK__RD, RDBK\n") ;
    fprintf (fp, "PIN, TRIG, I, LN__RD2\n") ;
    fprintf (fp, "PIN, DATA, O, LN__RD3\n") ;
    fprintf (fp, "END\n") ;
    fprintf (fp, "SYM, OBUF__RD, OBUF\n") ;
    fprintf (fp, "PIN, I,    I, LN__RD3\n") ;
    fprintf (fp, "PIN, O,    O, LN__RD4\n") ;
    fprintf (fp, "END\n") ;
    fprintf (fp, "SYM, MD1__RD, MD1\n") ;
    fprintf (fp, "PIN, O,    I, LN__RD4\n") ;
    fprintf (fp, "END\n") ;

	// Boundaryscan

    fprintf (fp, "SYM, TDI__BS, TDI, BLKNM=BS_TDI\n") ;
    fprintf (fp, "PIN, I,    O, LN__BS1\n") ;
    fprintf (fp, "END\n") ;
    fprintf (fp, "SYM, TCK__BS, TCK, BLKNM=BS_TCK\n") ;
    fprintf (fp, "PIN, I,    O, LN__BS2\n") ;
    fprintf (fp, "END\n") ;
    fprintf (fp, "SYM, TMS__BS, TMS, BLKNM=BS_TMS\n") ;
    fprintf (fp, "PIN, I,    O, LN__BS3\n") ;
    fprintf (fp, "END\n") ;
    fprintf (fp, "SYM, IBUF__BS1, IBUF\n") ;
    fprintf (fp, "PIN, I,    I, LN__BS1\n") ;
    fprintf (fp, "PIN, O,    O, LN__BS4\n") ;
    fprintf (fp, "END\n") ;
    fprintf (fp, "SYM, IBUF__BS2, IBUF\n") ;
    fprintf (fp, "PIN, I,    I, LN__BS2\n") ;
    fprintf (fp, "PIN, O,    O, LN__BS5\n") ;
    fprintf (fp, "END\n") ;
    fprintf (fp, "SYM, IBUF__BS3, IBUF\n") ;
    fprintf (fp, "PIN, I,    I, LN__BS3\n") ;
    fprintf (fp, "PIN, O,    O, LN__BS6\n") ;
    fprintf (fp, "END\n") ;
    fprintf (fp, "SYM, BSCAN__BS, BSCAN\n") ;
    fprintf (fp, "PIN, TDI,  I, LN__BS4\n") ;
    fprintf (fp, "PIN, TCK,  I, LN__BS5\n") ;
    fprintf (fp, "PIN, TMS,  I, LN__BS6\n") ;
    fprintf (fp, "PIN, TDO,  O, LN__BS7\n") ;
    fprintf (fp, "END\n") ;
    fprintf (fp, "SYM, TDO__BS, TDO\n") ;
    fprintf (fp, "PIN, O,    I, LN__BS7\n") ;
    fprintf (fp, "END\n") ;
}

void
write_inclock(FILE *fp, int c_rate)
{
	fprintf (fp, "SYM, INTOSC, OSC4\n") ;
	switch (c_rate) {
	case 15 :
		fprintf (fp, "PIN, F15, O, m_clock\n") ;
		break ;
	case 150 :
		fprintf (fp, "PIN, F15K, O, m_clock\n") ;
		break ;
	case 500 :
		fprintf (fp, "PIN, F500K, O, m_clock\n") ;
		break ;
	case 8 :
		fprintf (fp, "PIN, F8M, O, m_clock\n") ;
		break ;
	default :
		fprintf (fp, "PIN, F8M, O, m_clock\n") ;
		break ;
	}
	fprintf (fp, "END\n") ;
}
